#include "maze.h"
#include <stdlib.h>
#include <time.h>

void CMaze::drawWall(int x, int y, bool isHorizontal, Color c)
{
    int sx = x * TILE_SIZE;
    int sy = y * TILE_SIZE;

    if (isHorizontal == true)
    {
        int ex = sx + TILE_SIZE - 1;
        int ey = sy + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
    else
    {
        int ex = sx + WALL_WIDTH - 1;
        int ey = sy + TILE_SIZE + WALL_WIDTH - 1;

        for (int yy = sy; yy <= ey; yy++)
            for (int xx = sx; xx <= ex; xx++)
                gfx.setPixel(xx, yy, c);
    }
}

// choose a random place to put a trunk
STile* CMaze::chooseTrunk(bool& isHorizontal)
{
    int x, y;
    int dir = rand() % 4;

    if (dir == 0)
    {
        isHorizontal = true;
        x = 0;
        y = 1 + (rand() % (MAZE_HEIGHT - 1));
    }
    else if (dir == 1)
    {
        isHorizontal = true;
        x = MAZE_WIDTH - 1;
        y = 1 + (rand() % (MAZE_HEIGHT - 1));
    }
    else if (dir == 2)
    {
        isHorizontal = false;
        x = 1 + (rand() % (MAZE_WIDTH - 1));
        y = 0;
    }
    else
    {
        isHorizontal = false;
        x = 1 + (rand() % (MAZE_WIDTH - 1));
        y = MAZE_HEIGHT - 1;
    }

    return getTile(x, y);
}

CMaze::CMaze()
{
    srand(time(NULL));

    // init the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            tiles[x][y].topIsActive  = false;
            tiles[x][y].leftIsActive = false;
        }

    // init left and right borders
    for (int y = 0; y < MAZE_HEIGHT; y++)
    {
        tiles[0][y].leftIsActive = true;
        tiles[0][y].leftColor = Color(255, 255, 255);

        tiles[MAZE_WIDTH][y].leftIsActive = true;
        tiles[MAZE_WIDTH][y].leftColor = Color(255, 255, 255);
    }

    // init top and bottom borders
    for (int x = 0; x < MAZE_WIDTH; x++)
    {
        tiles[x][0].topIsActive = true;
        tiles[x][0].topColor = Color(255, 255, 255);

        tiles[x][MAZE_HEIGHT].topColor = Color(255, 255, 255);
        tiles[x][MAZE_HEIGHT].topIsActive = true;
    }

    // init the trunks
    for (int i = 0; i < NB_TRUNKS; ++i)
    {
        bool isHorizontal;
        Color   color(32 + rand() % 224,
                      32 + rand() % 224,
                      32 + rand() % 224);

        while (true)
        {
            STile*  tile = chooseTrunk(isHorizontal);

            if (isHorizontal == true)
            {
                if (tile->topIsActive == false)
                {
                    tile->topIsActive = true;
                    tile->topColor = color;
                    break;
                }
            }
            else
            {
                if (tile->leftIsActive == false)
                {
                    tile->leftIsActive = true;
                    tile->leftColor = color;
                    break;
                }
            }
        }
    }
}

void CMaze::draw()
{
    // draw the maze
    for (int y = 0; y <= MAZE_HEIGHT; y++)
        for (int x = 0; x <= MAZE_WIDTH; x++)
        {
            if (tiles[x][y].leftIsActive == true)
                drawWall(x, y, false, tiles[x][y].leftColor);

            if (tiles[x][y].topIsActive == true)
                drawWall(x, y, true, tiles[x][y].topColor);
        }
}
